<?php
/**
 * @package     SSL Fixer
 * @version     1.1
 * @author      Stergios Zgouletas <info@web-expert.gr>
 * @link        http://www.web-expert.gr
 * @copyright   Copyright (C) 2010 Web-Expert.gr All Rights Reserved
 * @license     http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */
defined( '_JEXEC' ) or die( 'Restricted access' );

jimport( 'joomla.plugin.plugin' );
jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');
//jimport( 'joomla.application.media.helper' );

class plgSystemSslfixer extends JPlugin
{
	protected $unsecureExt=array('html','htm','php', 'phps', 'pht', 'phtml', 'php3', 'php4', 'php5', 'php6', 'php7', 'inc', 'pl', 'cgi', 'fcgi', 'java', 'jar', 'py', 'inc', 'phps', 'class', 'php3', 'php4', 'php5', 'txt', 'dat', 'tpl', 'tmpl');

	function onAfterInitialise()
	{
		$app =JFactory::getApplication();
		if($app->isAdmin() || !$app->isSSLConnection() || $app->input->get('plg')!='sslfixer' || empty($app->input->getString('url',''))) return;		
		
		//$mediaHelper = new JHelperMedia;
		
		$url=urldecode($app->input->getString('url'));
		$urlParts=@explode('?',$url);
		$host=parse_url($urlParts[0],PHP_URL_HOST);
		$ext=JString::strtolower(pathinfo($urlParts[0],PATHINFO_EXTENSION));

		//securty check first
		if(empty($ext) || in_array($ext,$this->unsecureExt)) return;
		
		$allowedExtensions=$this->toArray($this->params->get('extensions',''),',');
		$hostList=$this->toArray($this->params->get('hosts',''),"\n");
		$mode=(int)$this->params->get('mode',0);
		
		$allowedHost=($mode==0 && (!count($hostList) || !in_array($host,$hostList))) || ($mode==1 && count($hostList) && in_array($host,$hostList));

		if($allowedHost && (!count($allowedExtensions) || in_array($ext,$allowedExtensions)))
		{
			$saveImages=$this->params->get('savelocaly',0);
			$mime=false;
			$data=$this->storeFile($url);
			if($data['result'])
			{
				$mime=$this->getMimeType($data['path'],$this->params->get('imageonly',0));						
				if($mime!==false)
				{
					$cacheTime=60*60*24*7;
					version_compare(JVERSION, '3.6', 'ge')? $app->clearHeaders():JResponse::clearHeaders();
					version_compare(JVERSION, '3.6', 'ge')? $app->allowCache(true):JResponse::allowCache(true);
					
					$this->setHeader('Content-Type',$mime.'; charset=utf-8',true);			
					$gmdate_mod = gmdate('D, d M Y H:i:s', filemtime($_SERVER['SCRIPT_FILENAME'])) . ' GMT';
					$this->setHeader('Last-Modified',$gmdate_mod,true);				
					$this->setHeader('Cache-Control','public, max-age='.$cacheTime.', pre-check='.$cacheTime.', post-check='.$cacheTime.', stale-while-revalidate='.($cacheTime*2).', stale-if-error='.($cacheTime*3),true);
					$this->setHeader('Pragma', 'public',true);
					$this->setHeader('Expires', gmdate('D, d M Y H:i:s', time()+$cacheTime).' GMT',true);
					$this->setHeader('X-Logged-In','False',true);
					version_compare(JVERSION, '3.6', 'ge')? $app->sendHeaders():JResponse::sendHeaders();
					readfile($data['path']);
				}
			}
			else
			{
				$app->redirect($url);
			}
			$app->close();
		}
	}
	
	function onAfterRender()
	{
		$app = JFactory::getApplication();
		$doc = JFactory::getDocument();
		$docType = $doc->getType();
		if($docType != 'html'  || $app->isAdmin() || !$app->isSSLConnection()) return;
		
		$allowedExtensions=$this->toArray($this->params->get('extensions',''),',');
		$sslHosts=$this->toArray($this->params->get('sslhosts',''),"\n");
		$hostList=$this->toArray($this->params->get('hosts',''),"\n");
		$mode=(int)$this->params->get('mode',0);
		$saveImages=$this->params->get('savelocaly',0);
		$saveImagePath=($saveImages==1?JPATH_CACHE:JPATH_ROOT.DIRECTORY_SEPARATOR.'images').DIRECTORY_SEPARATOR.'remote_images'.DIRECTORY_SEPARATOR;

		$buffer = version_compare(JVERSION, '3.6', 'ge')?$app->getBody():JResponse::getBody();
		@preg_match_all('/src="(http:.*)"/isU',$buffer,$matchesSRC);
		@preg_match_all('/<link(.*)href="(http:.*)"/isU',$buffer,$matchesHREF);
		$urls=@array_merge($matchesSRC[1],$matchesHREF[2]);
		$urls=@array_unique($urls);
		if(!count($urls)) return ;
		$debug=array();
		foreach($urls as $urlOld)
		{
			$urlNew=$urlOld;
			if(JURI::isInternal($urlOld))
			{
				$urlNew=str_replace('http:','https:',$urlOld);
			}
			else
			{
				$cleanURL=$urlOld;
				if(JString::strpos($urlOld,'?')!==false) list($cleanURL)=@explode('?',$urlOld);
				$host=parse_url($cleanURL,PHP_URL_HOST);
				$ext=JString::strtolower(pathinfo($cleanURL,PATHINFO_EXTENSION));
				if(empty($ext) || in_array($ext,$this->unsecureExt)) continue;
				$allowedHost=($mode==0 && (!count($hostList) || !in_array($host,$hostList))) || ($mode==1 && count($hostList) && in_array($host,$hostList));
				$image=md5($urlOld).'.'.$ext;
				
				if(in_array($host,$sslHosts))
				{
					$urlNew=str_replace('http:','https:',$urlOld);
				}
				elseif($saveImages && JFile::exists($saveImagePath.$image) && filesize($saveImagePath.$image)>0)
				{
					$urlNew=JURI::root(true).'/'.($saveImages==1?'cache':'images').'/remote_images/'.$image;
				}
				elseif($allowedHost && (!count($allowedExtensions) || in_array($ext,$allowedExtensions)))
				{
					$urlNew=JURI::root(true).'/index.php?plg=sslfixer&url='.urlencode($urlOld);				
				}
			}
			$count=0;
			if($urlNew!=$urlOld) $buffer=str_replace($urlOld,$urlNew,$buffer,$count);
			$debug[]='Host:'.$host.' URL Found:'.$urlOld.' Ext:'.$ext.' New SSL URL:'.$urlNew. ' Replaced:'.$count. ' HostCheck:'.$allowedHost;
		}
		
		if($this->params->get('debug',0))
		{
			$buffer=str_replace('</body>','<div style="border:2px solid red;margin:10px;"><h3>SSL Fixer Debug</h3><br/>'.implode('<br/>',$debug).'</div></body>',$buffer);
		}
		
		version_compare(JVERSION, '3.6', 'ge')?$app->setBody($buffer):JResponse::setBody($buffer);
	}

	protected function setHeader($name,$value,$override=false)
	{
		if (version_compare(JVERSION, '3.6', 'ge'))
		{
			JFactory::getApplication()->setHeader($name,$value,$override);
		} 
		else
		{
			JResponse::setHeader($name,$value,$override);
		}
	}
	
	//param to array
	protected function toArray($param,$sep=',')
	{
		$param=str_replace(array("\r\n", "\r"), $sep, JString::trim($param));
		$paramArray=array_unique(array_filter(explode($sep,$param)));
		return array_map('JString::trim',$paramArray);
	}
	
	protected function getMimeType($file, $isImage = false)
	{
		// If we can't detect anything mime is false
		$mime = false;

		try
		{
			if ($isImage && function_exists('exif_imagetype'))
			{
				$mime = image_type_to_mime_type(exif_imagetype($file));
			}
			elseif ($isImage && function_exists('getimagesize'))
			{
				$imagesize = getimagesize($file);
				$mime      = isset($imagesize['mime']) ? $imagesize['mime'] : false;
			}
			elseif (function_exists('mime_content_type'))
			{
				// We have mime magic.
				$mime = mime_content_type($file);
			}
			elseif (function_exists('finfo_open'))
			{
				// We have fileinfo
				$finfo = finfo_open(FILEINFO_MIME_TYPE);
				$mime  = finfo_file($finfo, $file);
				finfo_close($finfo);
			}
		}
		catch (\Exception $e)
		{
			// If we have any kind of error here => false;
			return false;
		}

		// If we can't detect the mime try it again
		if ($mime === 'application/octet-stream' && $isImage === true)
		{
			$mime = $this->getMimeType($file, false);
		}

		// We have a mime here
		return $mime;
	}
	
	
	protected function storeFile($url)
	{
		$urlParts=@explode('?',$url);
		$urlRaw=isset($urlParts[0])?$urlParts[0]:$url;
		$ext=JString::strtolower(pathinfo($urlRaw,PATHINFO_EXTENSION));
		
		$saveImages=$this->params->get('savelocaly',0);
		$path=$saveImages==1?JPATH_CACHE:JPATH_ROOT.DIRECTORY_SEPARATOR.'images';
		$saveImagePath=$path.DIRECTORY_SEPARATOR.'remote_images'.DIRECTORY_SEPARATOR;
		$image=md5($url).'.'.$ext;
		
		if(!JFolder::exists($saveImagePath) || !is_dir($saveImagePath)) JFolder::create($saveImagePath);
				
		if(!JFile::exists($saveImagePath.$image) || !filesize($saveImagePath.$image))
		{
			$buffer=@file_get_contents($url);
			if($buffer===false || !JFile::write($saveImagePath.$image,$buffer))
			{
				return array('result'=>false);
			}
		}
		
		//$cleanURL=str_replace(JPATH_ROOT,JURI::root(true),$saveImagePath.$image);
		return array('result'=>true,'path'=>$saveImagePath.$image); 	
	}
}